﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Modbus.Device;
using System.IO.Ports;

namespace NModbusTest
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        ModbusSerialMaster _modbusMaster;
        SerialPort _serialPort;
        const int SLAVE_ADDRESS = 1;

        private void Form1_Load(object sender, EventArgs e)
        {
            _serialPort = new SerialPort("COM1", 115200);
            _serialPort.Open();

            _modbusMaster = ModbusSerialMaster.CreateRtu(_serialPort);
            _modbusMaster.Transport.ReadTimeout = 500;
            _modbusMaster.Transport.WriteTimeout = 500;
            _modbusMaster.Transport.Retries = 0;

        }

        void ReportError(string s)
        {
            MessageBox.Show(s);
        }

        private void ReadADC()
        {
            try
            {
                ushort[] result = _modbusMaster.ReadInputRegisters(SLAVE_ADDRESS, 276, 1);
                _adcLabel.Text = result[0].ToString();
            }
            catch (Exception ex)
            {
                ReportError(ex.Message);
            }
        }

        private void ReadDigitalOutput()
        {
            try
            {
                bool[] result = _modbusMaster.ReadCoils(SLAVE_ADDRESS, 32, 1);
                if (result[0])
                {
                    _digitalOutLabel.Text = "ON";
                }
                else
                {
                    _digitalOutLabel.Text = "OFF";
                }
            }
            catch (Exception ex)
            {
                ReportError(ex.Message);
            }
        }

        private void WriteDigitalOutput(bool on)
        {
            try
            {
                _modbusMaster.WriteSingleCoil(SLAVE_ADDRESS, 32, on);
            }
            catch (Exception ex)
            {
                ReportError(ex.Message);
            }
        }

        private void _digitalOutReadButton_Click(object sender, EventArgs e)
        {
            ReadDigitalOutput();
        }

        private void _onButton_Click(object sender, EventArgs e)
        {
            WriteDigitalOutput(true);
        }

        private void _offButton_Click(object sender, EventArgs e)
        {
            WriteDigitalOutput(false);
        }

        private void _adcReadButton_Click(object sender, EventArgs e)
        {
            ReadADC();
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            _modbusMaster.Dispose();
            _modbusMaster = null;

            _serialPort.Close();
            _serialPort.Dispose();
            _serialPort = null;
        }
    }
}